import * as tapable from "tapable";
import { Source } from "webpack-sources";
import { JsCompilation, AssetInfo } from "@rspack/binding";
import { RspackOptionsNormalized } from "./config";
export declare class Compilation {
    #private;
    hooks: {
        processAssets: tapable.AsyncSeriesHook<Record<string, Source>>;
    };
    fullHash: string;
    hash: string;
    options: RspackOptionsNormalized;
    constructor(options: RspackOptionsNormalized, inner: JsCompilation);
    /**
     * Get a map of all assets.
     *
     * Source: [assets](https://github.com/webpack/webpack/blob/9fcaa243573005d6fdece9a3f8d89a0e8b399613/lib/Compilation.js#L1008-L1009)
     */
    get assets(): Record<string, Source>;
    /**
     * Update an existing asset. Trying to update an asset that doesn't exist will throw an error.
     *
     * See: [Compilation.updateAsset](https://webpack.js.org/api/compilation-object/#updateasset)
     * Source: [updateAsset](https://github.com/webpack/webpack/blob/9fcaa243573005d6fdece9a3f8d89a0e8b399613/lib/Compilation.js#L4320)
     *
     * FIXME: *AssetInfo* may be undefined in update fn for webpack impl, but still not implemented in rspack
     *
     * @param {string} file file name
     * @param {Source | function(Source): Source} newSourceOrFunction new asset source or function converting old to new
     * @param {AssetInfo | function(AssetInfo): AssetInfo} assetInfoUpdateOrFunction new asset info or function converting old to new
     */
    updateAsset(filename: string, newSourceOrFunction: Source | ((source: Source) => Source), assetInfoUpdateOrFunction: AssetInfo | ((assetInfo: AssetInfo) => AssetInfo)): void;
    /**
     * Emit an not existing asset. Trying to emit an asset that already exists will throw an error.
     *
     * See: [Compilation.emitAsset](https://webpack.js.org/api/compilation-object/#emitasset)
     * Source: [emitAsset](https://github.com/webpack/webpack/blob/9fcaa243573005d6fdece9a3f8d89a0e8b399613/lib/Compilation.js#L4239)
     *
     * @param {string} file file name
     * @param {Source} source asset source
     * @param {AssetInfo} assetInfo extra asset information
     * @returns {void}
     */
    emitAsset(filename: string, source: Source, assetInfo?: AssetInfo): void;
    /**
     * Get an array of Asset
     *
     * See: [Compilation.getAssets](https://webpack.js.org/api/compilation-object/#getassets)
     * Source: [getAssets](https://github.com/webpack/webpack/blob/9fcaa243573005d6fdece9a3f8d89a0e8b399613/lib/Compilation.js#L4448)
     *
     * @return {Readonly<Asset>[]}
     */
    getAssets(): {
        source: Source;
        name: string;
        info: AssetInfo;
    }[];
    getPath(filename: string, data?: Record<string, any>): any;
    getAssetPath(filename: any, data: any): any;
    createStats(): {};
    seal(): void;
    unseal(): void;
}
//# sourceMappingURL=compilation.d.ts.map