"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _Compiler_instances, _Compiler_instance_get, _Compiler_done, _Compiler_processAssets, _Compiler_compilation, _Compiler_newCompilation;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Compiler = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const util_1 = __importDefault(require("util"));
const tapable = __importStar(require("tapable"));
const tapable_1 = require("tapable");
const neo_async_1 = __importDefault(require("neo-async"));
const binding = __importStar(require("@rspack/binding"));
const Logger_1 = require("./logging/Logger");
const watch_1 = require("./config/watch");
const stats_1 = require("./stats");
const compilation_1 = require("./compilation");
const createSource_1 = require("./utils/createSource");
class EntryPlugin {
    apply() { }
}
class HotModuleReplacementPlugin {
    apply() { }
}
class Compiler {
    constructor(context, options) {
        _Compiler_instances.add(this);
        this.options = options;
        // to workaround some plugin access webpack, we may change dev-server to avoid this hack in the future
        this.webpack = {
            EntryPlugin,
            HotModuleReplacementPlugin // modernjs/server will auto inject this this plugin not set
        };
        this.hooks = {
            initialize: new tapable_1.SyncHook([]),
            done: new tapable.AsyncSeriesHook(["stats"]),
            afterDone: new tapable.SyncHook(["stats"]),
            beforeRun: new tapable.AsyncSeriesHook(["compiler"]),
            run: new tapable.AsyncSeriesHook(["compiler"]),
            thisCompilation: new tapable.SyncHook([
                "compilation"
                // "params"
            ]),
            compilation: new tapable.SyncHook(["compilation"]),
            invalid: new tapable_1.SyncHook(["filename", "changeTime"]),
            compile: new tapable_1.SyncHook(["params"]),
            infrastructureLog: new tapable_1.SyncBailHook(["origin", "type", "args"]),
            failed: new tapable_1.SyncHook(["error"])
        };
    }
    getInfrastructureLogger(name) {
        if (!name) {
            throw new TypeError("Compiler.getInfrastructureLogger(name) called without a name");
        }
        return new Logger_1.Logger((type, args) => {
            if (typeof name === "function") {
                name = name();
                if (!name) {
                    throw new TypeError("Compiler.getInfrastructureLogger(name) called with a function not returning a name");
                }
            }
            else {
                if (this.hooks.infrastructureLog.call(name, type, args) === undefined) {
                    if (this.infrastructureLogger !== undefined) {
                        this.infrastructureLogger(name, type, args);
                    }
                }
            }
        }, childName => {
            if (typeof name === "function") {
                if (typeof childName === "function") {
                    return this.getInfrastructureLogger(_ => {
                        if (typeof name === "function") {
                            name = name();
                            if (!name) {
                                throw new TypeError("Compiler.getInfrastructureLogger(name) called with a function not returning a name");
                            }
                        }
                        if (typeof childName === "function") {
                            childName = childName();
                            if (!childName) {
                                throw new TypeError("Logger.getChildLogger(name) called with a function not returning a name");
                            }
                        }
                        return `${name}/${childName}`;
                    });
                }
                else {
                    return this.getInfrastructureLogger(() => {
                        if (typeof name === "function") {
                            name = name();
                            if (!name) {
                                throw new TypeError("Compiler.getInfrastructureLogger(name) called with a function not returning a name");
                            }
                        }
                        return `${name}/${childName}`;
                    });
                }
            }
            else {
                if (typeof childName === "function") {
                    return this.getInfrastructureLogger(() => {
                        if (typeof childName === "function") {
                            childName = childName();
                            if (!childName) {
                                throw new TypeError("Logger.getChildLogger(name) called with a function not returning a name");
                            }
                        }
                        return `${name}/${childName}`;
                    });
                }
                else {
                    return this.getInfrastructureLogger(`${name}/${childName}`);
                }
            }
        });
    }
    run(callback) {
        const doRun = () => {
            const finalCallback = (err, stats) => {
                if (err) {
                    this.hooks.failed.call(err);
                }
                if (callback) {
                    callback(err, stats);
                }
                this.hooks.afterDone.call(stats);
            };
            this.hooks.beforeRun.callAsync(this, err => {
                if (err) {
                    return finalCallback(err);
                }
                this.hooks.run.callAsync(this, err => {
                    if (err) {
                        return finalCallback(err);
                    }
                    this.build((err, rawStats) => {
                        if (err) {
                            return finalCallback(err);
                        }
                        const stats = new stats_1.Stats(rawStats);
                        this.hooks.done.callAsync(stats, err => {
                            if (err) {
                                return finalCallback(err);
                            }
                            else {
                                return finalCallback(null, stats);
                            }
                        });
                    });
                });
            });
        };
        doRun();
    }
    // Safety: This method is only valid to call if the previous build task is finished, or there will be data races.
    build(cb) {
        const build_cb = __classPrivateFieldGet(this, _Compiler_instances, "a", _Compiler_instance_get).unsafe_build.bind(__classPrivateFieldGet(this, _Compiler_instances, "a", _Compiler_instance_get));
        build_cb((err, stats) => {
            if (err) {
                cb(err);
            }
            else {
                cb(null, stats);
            }
        });
    }
    // Safety: This method is only valid to call if the previous rebuild task is finished, or there will be data races.
    rebuild(changedFiles, cb) {
        const rebuild_cb = __classPrivateFieldGet(this, _Compiler_instances, "a", _Compiler_instance_get).unsafe_rebuild.bind(__classPrivateFieldGet(this, _Compiler_instances, "a", _Compiler_instance_get));
        rebuild_cb(changedFiles, [], (err, stats) => {
            if (err) {
                cb(err);
            }
            else {
                cb(null, stats);
            }
        });
    }
    // TODO: use ws to send message to client temporary.
    // TODO: we should use `Stats` which got from `hooks.done`
    // TODO: in `dev-server`
    async watch(watchOptions, ws) {
        const options = (0, watch_1.resolveWatchOption)(watchOptions);
        let logger = this.getInfrastructureLogger("watch");
        const watcher = (await Promise.resolve().then(() => __importStar(require("chokidar")))).default.watch(this.options.context, {
            ignoreInitial: true,
            ...options
        });
        const begin = Date.now();
        let rawStats = await util_1.default.promisify(this.build.bind(this))();
        let stats = new stats_1.Stats(rawStats);
        // TODO: log stats string should move to cli
        console.log(stats.toString(this.options.stats));
        console.log("build success, time cost", Date.now() - begin, "ms");
        let pendingChangedFilepaths = new Set();
        let isBuildFinished = true;
        // TODO: should use aggregated
        watcher.on("change", async (changedFilepath) => {
            // TODO: only build because we lack the snapshot info of file.
            // TODO: it means there a lot of things to do....
            // store the changed file path, it may or may not be consumed right now
            if (!isBuildFinished) {
                pendingChangedFilepaths.add(changedFilepath);
                console.log("hit change but rebuild is not finished, caching files: ", pendingChangedFilepaths);
                return;
            }
            const rebuildWithFilepaths = (changedFilepath) => {
                // Rebuild finished, we can start to rebuild again
                isBuildFinished = false;
                console.log("hit change and start to build");
                const begin = Date.now();
                this.rebuild(changedFilepath, (error, { diff, stats: rawStats }) => {
                    let stats = new stats_1.Stats(rawStats);
                    // TODO: log stats string should move to cli
                    console.log(stats.toString(this.options.stats));
                    isBuildFinished = true;
                    const hasPending = Boolean(pendingChangedFilepaths.size);
                    // If we have any pending task left, we should rebuild again with the pending files
                    if (hasPending) {
                        const pending = [...pendingChangedFilepaths];
                        pendingChangedFilepaths.clear();
                        rebuildWithFilepaths(pending);
                    }
                    if (error) {
                        throw error;
                    }
                    for (const [uri, stats] of Object.entries(diff)) {
                        let relativePath = path_1.default.relative(this.options.context, uri);
                        if (!(relativePath.startsWith("../") ||
                            relativePath.startsWith("./"))) {
                            relativePath = "./" + relativePath;
                        }
                        // send Message
                        if (ws) {
                            const data = {
                                uri: relativePath,
                                content: stats.content
                            };
                            if (/\.[less|css|sass|scss]$/.test(data.uri)) {
                                const cssOutput = fs_1.default
                                    .readFileSync(path_1.default.resolve(this.options.output.path, "main.css"))
                                    .toString("utf-8");
                                // TODO: need support more
                                data.content = [
                                    `var cssStyleTag = document.querySelector("style[id='hot-css']")`,
                                    `if (cssStyleTag) {`,
                                    `  cssStyleTag.innerText = \`${cssOutput}\``,
                                    `} else {`,
                                    `  var newCSStyleTag = document.createElement('style')`,
                                    `  newCSStyleTag.setAttribute('id', 'hot-css')`,
                                    `  newCSStyleTag.innerText = \`${cssOutput}\``,
                                    `  document.head.appendChild(newCSStyleTag)`,
                                    `}`,
                                    ``,
                                    `var outdataCSSLinkTag = document.querySelector("link[href='main.css']")`,
                                    `outdataCSSLinkTag && outdataCSSLinkTag.parentNode && outdataCSSLinkTag.parentNode.removeChild(outdataCSSLinkTag)`
                                ].join("\n");
                            }
                            for (const client of ws.clients) {
                                // the type of "ok" means rebuild success.
                                // the data should deleted after we had hash in stats.
                                client.send(JSON.stringify({ type: "ok", data: JSON.stringify(data) }));
                            }
                        }
                    }
                    console.log("rebuild success, time cost", Date.now() - begin, "ms");
                });
            };
            rebuildWithFilepaths([...pendingChangedFilepaths, changedFilepath]);
        });
        return {
            async close() {
                await watcher.close();
            }
        };
    }
    purgeInputFileSystem() {
        if (this.inputFileSystem && this.inputFileSystem.purge) {
            this.inputFileSystem.purge();
        }
    }
    /**
     * @todo
     */
    close(callback) {
        callback();
    }
    emitAssets(compilation, callback) {
        const outputPath = compilation.getPath(this.outputPath, {});
        fs_1.default.mkdirSync(outputPath, { recursive: true });
        const assets = compilation.getAssets();
        neo_async_1.default.forEachLimit(assets, 15, ({ name: file, source, info }, callback) => {
            let targetFile = file;
            const absPath = path_1.default.resolve(outputPath, targetFile);
            const getContent = () => {
                if (typeof source.buffer === "function") {
                    return source.buffer();
                }
                else {
                    const bufferOrString = source.source();
                    if (Buffer.isBuffer(bufferOrString)) {
                        return bufferOrString;
                    }
                    else {
                        return Buffer.from(bufferOrString, "utf-8");
                    }
                }
            };
            const doWrite = content => {
                this.outputFileSystem.writeFile(absPath, content, callback);
            };
            let content = getContent();
            doWrite(content);
        });
    }
}
exports.Compiler = Compiler;
_Compiler_instances = new WeakSet(), _Compiler_instance_get = function _Compiler_instance_get() {
    // @ts-ignored
    this._instance =
        // @ts-ignored
        this._instance ||
            // @ts-ignored
            new binding.Rspack(this.options, {
                doneCallback: __classPrivateFieldGet(this, _Compiler_instances, "m", _Compiler_done).bind(this),
                processAssetsCallback: __classPrivateFieldGet(this, _Compiler_instances, "m", _Compiler_processAssets).bind(this),
                // `Compilation` should be created with hook `thisCompilation`, and here is the reason:
                // We know that the hook `thisCompilation` will not be called from a child compiler(it doesn't matter whether the child compiler is created on the Rust or the Node side).
                // See webpack's API: https://webpack.js.org/api/compiler-hooks/#thiscompilation
                // So it is safe to create a new compilation here.
                thisCompilationCallback: __classPrivateFieldGet(this, _Compiler_instances, "m", _Compiler_newCompilation).bind(this),
                // The hook `Compilation` should be called whenever it's a call from the child compiler or normal compiler and
                // still it does not matter where the child compiler is created(Rust or Node) as calling the hook `compilation` is a required task.
                // No matter how it will be implemented, it will be copied to the child compiler.
                compilationCallback: __classPrivateFieldGet(this, _Compiler_instances, "m", _Compiler_compilation).bind(this)
            });
    // @ts-ignored
    return this._instance;
}, _Compiler_done = function _Compiler_done(statsJson) { }, _Compiler_processAssets = async function _Compiler_processAssets(assets) {
    let iterator = Object.entries(assets).map(([filename, source]) => [
        filename,
        (0, createSource_1.createSourceFromRaw)(source)
    ]);
    await this.compilation.hooks.processAssets.promise(Object.fromEntries(iterator));
}, _Compiler_compilation = function _Compiler_compilation(native) {
    // TODO: implement this based on the child compiler impl.
    this.hooks.compilation.call(this.compilation);
}, _Compiler_newCompilation = function _Compiler_newCompilation(native) {
    const compilation = new compilation_1.Compilation(this.options, native);
    this.compilation = compilation;
    this.hooks.thisCompilation.call(this.compilation);
};
//# sourceMappingURL=compiler.js.map