"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveBuiltinsOptions = void 0;
const browserslist_1 = require("browserslist");
function resolveDefine(define = {}) {
    const entries = Object.entries(define).map(([key, value]) => [
        key,
        JSON.stringify(value)
    ]);
    return Object.fromEntries(entries);
}
function resolveHtml(html) {
    return html.map(c => {
        for (const key in c.meta) {
            const value = c.meta[key];
            if (typeof value === "string") {
                c.meta[key] = {
                    name: key,
                    content: value
                };
            }
        }
        return c;
    });
}
function resolveDecorator(decorator) {
    if (decorator === false) {
        return undefined;
    }
    if (decorator === undefined || decorator === true) {
        decorator = {};
    }
    return Object.assign({
        legacy: true,
        emitMetadata: true,
        useDefineForClassFields: true
    }, decorator);
}
function resolveBuiltinsOptions(builtins, contextPath) {
    const browserslist = (0, browserslist_1.loadConfig)({ path: contextPath }) || [];
    return {
        ...builtins,
        html: resolveHtml(builtins.html || []),
        browserslist,
        decorator: resolveDecorator(builtins.decorator)
    };
}
exports.resolveBuiltinsOptions = resolveBuiltinsOptions;
//# sourceMappingURL=builtins.js.map