"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyRspackOptionsDefaults = exports.applyRspackOptionsBaseDefaults = void 0;
const path_1 = __importDefault(require("path"));
const target_1 = require("./target");
const D = (obj, prop, value) => {
    if (obj[prop] === undefined) {
        obj[prop] = value;
    }
};
const F = (obj, prop, factory) => {
    if (obj[prop] === undefined) {
        obj[prop] = factory();
    }
};
function applyRspackOptionsBaseDefaults(options) {
    F(options, "context", () => process.cwd());
    applyInfrastructureLoggingDefaults(options.infrastructureLogging);
}
exports.applyRspackOptionsBaseDefaults = applyRspackOptionsBaseDefaults;
/**
 * @param {InfrastructureLogging} infrastructureLogging options
 * @returns {void}
 */
const applyInfrastructureLoggingDefaults = infrastructureLogging => {
    F(infrastructureLogging, "stream", () => process.stderr);
    const tty = 
    /** @type {any} */ infrastructureLogging.stream.isTTY &&
        process.env.TERM !== "dumb";
    D(infrastructureLogging, "level", "info");
    D(infrastructureLogging, "debug", false);
    D(infrastructureLogging, "colors", tty);
    D(infrastructureLogging, "appendOnly", !tty);
};
const applyOutputDefault = (output) => {
    D(output, "hashFunction", "xxhash64");
    F(output, "path", () => path_1.default.resolve(process.cwd(), "dist"));
    return output;
};
function applyRspackOptionsDefaults(options) {
    F(options, "context", () => process.cwd());
    /** @todo  */
    F(options, "target", () => {
        return (0, target_1.getDefaultTarget)(options.context);
    });
    const { mode, context } = options;
    const development = mode === "development";
    const production = mode === "production" || !mode;
    F(options, "devtool", () => (development ? "eval" : ""));
    applyOutputDefault(options.output);
    D(options.builtins, "minify", production);
}
exports.applyRspackOptionsDefaults = applyRspackOptionsDefaults;
//# sourceMappingURL=defaults.js.map