/// <reference types="node" />
import type { RawModuleRuleUse, RawModuleRule } from "@rspack/binding";
import { ResolvedContext } from "./context";
import { ResolvedDevtool } from "./devtool";
export interface ModuleRule {
    name?: string;
    test?: RawModuleRule["test"];
    resource?: RawModuleRule["resource"];
    resourceQuery?: RawModuleRule["resourceQuery"];
    uses?: ModuleRuleUse[];
    type?: RawModuleRule["type"];
}
export interface Module {
    rules?: ModuleRule[];
    parser?: {
        dataUrlCondition?: {
            maxSize?: number;
        };
    };
}
interface ResolvedModuleRule {
    test?: RawModuleRule["test"];
    resource?: RawModuleRule["resource"];
    resourceQuery?: RawModuleRule["resourceQuery"];
    uses?: RawModuleRuleUse[];
    type?: RawModuleRule["type"];
}
export interface ResolvedModule {
    rules: ResolvedModuleRule[];
    parser?: {
        dataUrlCondition: {
            maxSize: number;
        };
    };
}
interface LoaderContextInternal {
    source: number[];
    sourceMap: string | null;
    resource: string;
    resourcePath: string;
    resourceQuery: string | null;
    resourceFragment: string | null;
}
interface LoaderResult {
    content: Buffer | string;
    meta: Buffer | string;
}
export interface LoaderContext extends Pick<LoaderContextInternal, "resource" | "resourcePath" | "resourceQuery" | "resourceFragment" | "sourceMap"> {
    source: {
        getCode(): string;
        getBuffer(): Buffer;
    };
    useSourceMap: boolean;
    rootContext: string;
    context: string;
    getOptions: () => unknown;
}
export interface ComposeJsUseOptions {
    devtool: ResolvedDevtool;
    context: ResolvedContext;
}
interface JsLoader {
    (this: LoaderContext, loaderContext: LoaderContext): Promise<LoaderResult | void> | LoaderResult | void;
    displayName?: string;
}
declare type BuiltinLoader = string;
declare type ModuleRuleUse = {
    builtinLoader: BuiltinLoader;
    options?: unknown;
    name?: string;
} | {
    loader: JsLoader;
    options?: unknown;
    name?: string;
};
export declare function createRawModuleRuleUses(uses: ModuleRuleUse[], options: ComposeJsUseOptions): RawModuleRuleUse[];
export declare function resolveModuleOptions(module: Module, options: ComposeJsUseOptions): ResolvedModule;
export {};
//# sourceMappingURL=module.d.ts.map