"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCompiler = exports.rspack = void 0;
const config_1 = require("./config");
const compiler_1 = require("./compiler");
const defaults_1 = require("./config/defaults");
const createConsoleLogger_1 = __importDefault(require("./logging/createConsoleLogger"));
const nodeConsole_1 = __importDefault(require("./node/nodeConsole"));
const util_1 = __importDefault(require("util"));
const rspackOptionsApply_1 = require("./rspackOptionsApply");
function createCompiler(userOptions) {
    const options = (0, config_1.getNormalizedRspackOptions)(userOptions);
    (0, defaults_1.applyRspackOptionsBaseDefaults)(options);
    const compiler = new compiler_1.Compiler(options.context, options);
    const { infrastructureLogging } = options;
    compiler.infrastructureLogger = (0, createConsoleLogger_1.default)({
        level: infrastructureLogging.level || "info",
        debug: infrastructureLogging.debug || false,
        console: infrastructureLogging.console ||
            (0, nodeConsole_1.default)({
                colors: infrastructureLogging.colors,
                appendOnly: infrastructureLogging.appendOnly,
                stream: infrastructureLogging.stream
            })
    });
    const logger = compiler.getInfrastructureLogger("config");
    logger.debug("RawOptions:", util_1.default.inspect(userOptions, { colors: true, depth: null }));
    if (Array.isArray(options.plugins)) {
        for (const plugin of options.plugins) {
            if (typeof plugin === "function") {
                plugin.call(compiler, compiler);
            }
            else {
                plugin.apply(compiler);
            }
        }
    }
    (0, defaults_1.applyRspackOptionsDefaults)(compiler.options);
    logger.debug("NormalizedOptions:", util_1.default.inspect(compiler.options, { colors: true, depth: null }));
    new rspackOptionsApply_1.RspackOptionsApply().process(compiler.options, compiler);
    compiler.hooks.initialize.call();
    return compiler;
}
exports.createCompiler = createCompiler;
function rspack(options, callback) {
    let compiler = createCompiler(options);
    if (callback) {
        compiler.run(callback);
        return compiler;
    }
    else {
        return compiler;
    }
}
exports.rspack = rspack;
//# sourceMappingURL=rspack.js.map